lm.component('iotThings', {
  templateUrl: 'comp/misc/iotThings.' + lmCacheTag + 'html',
  bindings: {},
  controller: function($element, $http, serviceLimboMedia, serviceNotification, serviceDeps, serviceModal) {
		var ctrl = this;
		ctrl.sensors = [];
		
		ctrl.daterange = {
			from: moment().subtract(1,'days'),
			to: moment().add(1,'hour')
		}
		
		ctrl.sensorsString = [];
		ctrl.sensorsCoordinate = [];
		
		ctrl.loadSensors = function() {
			$http.get(lmBase.url + 'iot/items').then(
				function success(resp) {
					for(x in resp.data) {
						var it = resp.data[x];
						
						/* initially checked */
						for(y in ctrl.settings.sensorsOn) {
							var sensorid = ctrl.settings.sensorsOn[y];
							if(sensorid == it.id) {
								it.checked = true;
							}
						}
						
						if(it.type == 'CONTROL_SELECT') {
							for(y in it.options) {
								var option = it.options[y];
								if(it.last && it.last.val == option.key) {
									it.selectedOption = option; 
								}
							}
						}
						
					}					
					ctrl.sensors = resp.data;
					ctrl.redrawAll();
				},
				serviceLimboMedia.handleHttpErrorNew
			);
		};
		
		ctrl.sensorEdit = function(sens,sts) {
			serviceModal.show('lm-dialog-thing-crup',{item: sens}).then(
				  function() { ctrl.loadSensors(); },
				  function() { }
			);
		};	
		
		ctrl.sensorDelete = function(sens) {
			serviceModal.show('lm-dialog-delete',{url: lmBase.url + 'iot/item/' + sens.id, name: sens.name}).then(
					function() { ctrl.loadSensors(); },
					function() {}
			);
		};
		
		ctrl.sensorPush = function(it) {
			serviceModal.show('lm-dialog-thing-push',{item: it}).then(
				  function() { },
				  function() { }
			);			
		};
		
		ctrl.sensorPull = function(it) {
			serviceModal.show('lm-dialog-thing-pull',{item: it}).then(
				  function() { ctrl.loadSensors(); },
				  function() { }
			);
		};	
		
		ctrl.sensorShare = function(sens) {
			serviceModal.show('lm-dialog-share-simple',{user: sens.x.user, url: lmBase.url + 'iot/item/' + sens.id + '/shares'}).then(
				  function() { ctrl.loadSensors(); },
				  function() { }
			);
		};
		
		ctrl.openAction = function(it) {
			serviceModal.show('lm-dialog-thing-action',{item: it}).then(
				  function() { ctrl.loadSensors(); },
				  function() { }
			);
		};	
		
		ctrl.sensorLoadData = function(sens) {
			var par = {
				from: ctrl.daterange.from.valueOf(),
				to: ctrl.daterange.to.valueOf()
			}
			$http.get(lmBase.url + 'iot/item/' + sens.id + '/rawdata',{params : par, targetSENS: sens}).then(
				function success(resp) {
					resp.config.targetSENS.data = resp.data;
					ctrl.redraw();
				},
				serviceLimboMedia.handleHttpErrorNew
			);
		}
		
		ctrl.redrawAll = function() {
			for(x in ctrl.sensors) {
				var sens = ctrl.sensors[x];
				if(sens.checked) {
					ctrl.sensorLoadData(sens);
				}
			}			
		}
		
		ctrl.toggleSensor = function(sens) {
			if(sens.checked) {
				ctrl.sensorLoadData(sens);
			}
			else {
				sens.data = null;
				ctrl.redraw();
			}
			ctrl.saveToggled();
		};		
		
		ctrl.redraw = function() {
			ctrl.sensorsString = [];
			ctrl.sensorsCoordinate = [];
			
			var sd = [];
			var yaxises = [];
			
			var axcount = 1;
			for(x in ctrl.sensors) {
				var sens = ctrl.sensors[x];
				if(sens.data) {
					if(sens.dataType == 'STRING') {
						ctrl.sensorsString.push(sens);
					}
					else if(sens.dataType == 'COORDINATE') {
						ctrl.sensorsCoordinate.push(sens);
					}
					else {
						var axisToUse = null;
						for(y in yaxises) {
							var axis = yaxises[y];
							if(('BOOLEAN' == axis.dataType && 'BOOLEAN' == sens.dataType) || axis.name == sens.unit) {
								axisToUse = axis;
							}
						}
						if(!axisToUse) {
							axisToUse = {
								name: sens.unit,
								dataType: sens.dataType,
								idx: axcount++
							}
							yaxises.push(axisToUse);
						}
						
						var graphdata = {
								data: sens.data,
								label: sens.name + ' ' + sens.unit,
								yaxis: axisToUse.idx
						};

						if("FIRE" == sens.dataType) {
							graphdata = {
									data: sens.data,
									label: sens.name + ' ' + sens.unit,
									points: { show: true },
									yaxis: axisToUse.idx
							};
						}
						
						sd.push(graphdata);
					}
				}
			}
			var el = $($element).find('.plotit');

			serviceDeps.load("plot").then(function() {
				var graph = $.plot(el, sd, {
					legend: {show:true, position:"nw",noColumns:1},
					lines: {show:true},
					xaxis: {
						mode: "time",
						timezone: "browser",
						timeformat: "%Y-%m-%d<br/>%H:%M:%S"
					},
					yaxes: yaxises,
					selection: { mode: "x" },
					tooltip: true,
					tooltipOpts: { content: "<h4>%s</h4> Value: %y.3<br/>%x" },
					grid: { hoverable: true }				
				});			
			});
		};
		
		ctrl.saveToggled = function() {
			ctrl.settings.sensorsOn = [];
			for(x in ctrl.sensors) {
				var it = ctrl.sensors[x];
				if(it.checked) {
					ctrl.settings.sensorsOn.push(it.id);
				}
			}
			
			ctrl.saveSettings();
		}
		
		/* CONTROLS */
		ctrl.controlDoFire = function(con) {
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', {}).then(
				function success(resp) {
					serviceNotification.ok('DONE');
					con.last = resp.data;
				},
				serviceLimboMedia.handleHttpErrorNew
			);		
		};
		ctrl.controlDoSwitch = function(con, valOn) {
			var req = {val: valOn};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req).then(
				function success(resp) {
					con.last = resp.data;
				},
				serviceLimboMedia.handleHttpErrorNew
			);		
		};

		ctrl.controlDoSelect = function(con, valOn) {
			var req = {val: valOn.key};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req).then(
				function success(resp) {
					serviceNotification.ok('DONE');
					con.last = resp.data;
				},
				serviceLimboMedia.handleHttpErrorNew
			);		
		};

		ctrl.onPercentSliderChange = function(con, val) {
			var req = {'val': val};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req).then(
				function success(resp) {
					serviceNotification.ok('DONE');
					con.last = resp.data;
				},
				serviceLimboMedia.handleHttpErrorNew
			);		
		}

		ctrl.controlDoNumber = function(con) {
			if(!(con.last && con.last.val))
				return;
			
			var req = {val: con.last.val};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req).then(
				function success(resp) {
					serviceNotification.ok('DONE');
					con.last = resp.data;
				},
				serviceLimboMedia.handleHttpErrorNew
			);		
		};

		ctrl.controlDoString = function(con) {
			if(!(con.last && con.last.val))
				return;
			var req = {val: con.last.val};
			$http.post(lmBase.url + 'iot/item/' + con.id + '/control', req).then(
				function success(resp) {
					serviceNotification.ok('DONE');
					con.last = resp.data;
				},
				serviceLimboMedia.handleHttpErrorNew
			);		
		};
		
		
		/* SETTINGS */
		ctrl.settings = new Object();

		ctrl.saveSettings = function() {
			serviceLimboMedia.saveRemoteSettings();
		};	
		
		ctrl.settings = serviceLimboMedia.getRemoteSettings('iot');
		if(ctrl.settings.sensorsOn == null) ctrl.settings.sensorsOn = [];
		ctrl.loadSensors();

	}
});
