lm.factory('serviceGlobalEvents', ['$rootScope', 'serviceNotification', '$translate', function($rootScope, serviceNotification, $translate) { 
	var service = new Object();
	
	/* Registry. Only one listener for filedrops at a time. Callback must provide: */
	/* onDdCheck() -> called on drag -> return {allowed: true/false, msg: 'I18N_KEY'}. */
	/* onDdDrop(dataTransfer) -> called on drop -> handle dropped stuff. */
	service.listener = null;
	service.registerListenerDataTransfer = function(valueListener) {
		service.listener = valueListener;
		if(!service.listener) {
			// cleanup in case a leave/drop listener failed before
			service.toastOff();
		}
	};

	/* Use toasts to notify user when about to drop files. */
	service.toastRef = null;

	service.toastOn = function(checkResult) {
		if(!service.toastRef) {
			service.toastRef = 'tmp';
			
			$rootScope.$apply(function() {
				$translate(checkResult.msg).then(function (translation) {
					service.toastRef = checkResult.allowed ? serviceNotification.toastTheInfo(translation, true) : serviceNotification.toastTheError(translation, true);
				});
			});
		}
	};
	
	service.toastOff = function() {
		if(service.toastRef) {
			var tmp = service.toastRef;
			service.toastRef = null;
			$rootScope.$apply(function() {
				serviceNotification.toastDone(tmp);
			});
		}
	};


	/* Native event callbacks. */
	service.onDragOver = function(event) {
		event.preventDefault();  
		event.stopPropagation();
		if(service.listener) {
			var checkresult = service.listener.onDdCheck();
			service.toastOn(checkresult);
		}
	};
	
	service.onDragLeave = function(event) {
		event.preventDefault();  
		event.stopPropagation();
		service.toastOff();
	};

	service.onDrop = function(event) {
		event.preventDefault();  
		event.stopPropagation();
		// service.logInfo("DragDrop", event);
		service.toastOff();
		if(service.listener) {
			service.listener.onDdDrop(event.dataTransfer);
		}
	};

	/* for logging and dev of item based file and folder uploads. */
	service.logInfo = function(tag, event) {
		var theFiles = [];
		var theFolders = [];
		var theOthers = [];

		var dt = event.dataTransfer;
		console.log(tag + " DataTransfer: ", dt);
		
		var fs = dt.files;
		console.log(tag + " Files: ", fs.length, fs);
		
		var its = dt.items;
		console.log(tag + " Items: ", its.length, its);
		
		/*
		for (var i = 0; i < fs.length; i++) {
			file = fs[i];
			console.log(tag + "File: ", i, file);
			
			console.log(tag + " Size: " + file.size);
			var sli = file.slice(0,1);
			console.log(tag + " Slice: " + sli);
			console.log(tag + " SliSi: " + sli.size);
			
			try {
				var reader = new FileReader();
				reader.onload = function () {
					console.log("Reader result: " + reader.result, reader);
				}
				reader.onerror = function() {
					console.log("Reader error: " + reader.result, reader);
				}
				
	            // attempt to access the first few bytes of the file, will throw an exception if a directory
	            reader.readAsBinaryString(sli);
	            // console.log(tag + " Read: ",read);
	        } catch( e ) {
	        	console.log(tag + " Failed: ", e);
	            // could not access contents, is a directory, skip
	        }
	        
			// !f.type && f.size%4096 == 0 && f.size <= 102400
		}
		*/
		
		/*
		var its = dt.items;
		console.log(tag + " Items: ", its.length, its);
		for (var i = 0; i < its.length; i++) {
			item = its[i];
			console.log(tag + "Item: ", i, item);
			
			// !f.type && f.size%4096 == 0 && f.size <= 102400
		}
		*/
	};


	/* bind to document to catch events on whole viewport */
	document.addEventListener('drop', service.onDrop);
	document.addEventListener('dragover', service.onDragOver);
	document.addEventListener('dragleave', service.onDragLeave);

	return service;
}]);
