lm.factory('serviceModal', [ '$q', function($q) {
	var service = new Object();
	
	/* Stack full of modal instances */
	service.stack = [];
	
	service.internalClose = function(valueInstance) {
		var idx = service.stack.indexOf(valueInstance);
		if(idx == -1) {
			console.log("Cannot close modal. Not found in stack.");
			return null;
		}
		var instance = service.stack[idx];
		service.stack.splice(idx,1);
		return instance;
	};
	
	/* Callback for modal component on dismiss/close without action */
	service.dismiss = function(valueInstance) {
		var instance = service.internalClose(valueInstance);
		if(instance) {
			instance.defProm.reject();
		}
	};
	
	/* Callback for modal component on success action with optional result */
	service.done = function(valueInstance, valueResult) {
		var instance = service.internalClose(valueInstance);
		if(instance) {
			instance.defProm.resolve(valueResult);
		}
	};

	/* Open modal */
	service.show = function(valueName, valueParams, valueClosePromise) {
		var instance = {
			name: valueName,
			params: valueParams ? valueParams : {},	// make sure it's never undefined.
			defProm: $q.defer(),
			closable: !!!valueClosePromise			// can be closed by user?
		};
		
		service.stack.push(instance);
		
		if(valueClosePromise) {
			valueClosePromise.then(
				function(result) { service.dismiss(instance); },
				function(error) { service.dismiss(instance); }
			);
		}

		return instance.defProm.promise;
	};

	return service;
}]);
